# -*- coding: utf-8 -*-
# Project : flacon
# File : track.py
# (c) Alexander Sokolov sokoloff.a@gmail.com
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.


import re
from settings import *
from flaconglobal import *

from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import *


class Track(QObject):
   #****************************************************************
   #
   #****************************************************************
    def __init__(self, trackSet):
        QObject.__init__(self, trackSet)
        self._trackSet = trackSet
        self._tags = {}
        self._trackNum = 0
        self._startTrackNum = 1

        self._progress = -1
        self._status  = Statuses.NotRunning
        self._resultFileNameCache = None
        self._resultFilePathCache = None

        self.connect(settings, SIGNAL("changed()"), self._settingsChanged)
        self._settingsChanged()


   #****************************************************************
   #
   #****************************************************************
    def trackSet(self):
        return self._trackSet


   #****************************************************************
   #
   #****************************************************************
    def index(self):
        return self._trackSet.indexOf(self)


   #****************************************************************
   #
   #****************************************************************
    def emitDataChanged(self):
        self._trackSet.trackDataChanged(self)


   #****************************************************************
   #
   #****************************************************************
    def tag(self, tagName):
        try:
            (value, encoded) = self._tags[tagName]
            if encoded:
                return value
            else:
                return self._decode(value)

        except KeyError, e:
            return QString()


   #****************************************************************
   #
   #****************************************************************
    def setTag(self, tagName, value, encoded=True):
        self._resultFileNameCache = None
        self._resultFilePathCache = None
        self._tags[tagName] = (value, encoded)
        self.emitDataChanged()


   #****************************************************************
   #
   #****************************************************************
    def _decode(self, value):
        if not isinstance(value, QString):
            return value

        codec = self._trackSet.textCodec()
        if (codec):
            return codec.toUnicode(value.toAscii())
        else:
            return value


   #****************************************************************
   #
   #****************************************************************
    def artist(self):
        return self.tag('PERFORMER')


   #****************************************************************
   #
   #****************************************************************
    def setArtist(self, value, encoded=True):
        self.setTag('PERFORMER', value, encoded)


   #****************************************************************
   #
   #****************************************************************
    def album(self):
        return self.tag('ALBUM')


   #****************************************************************
   #
   #****************************************************************
    def setAlbum(self, value, encoded=True):
        self.setTag('ALBUM', value, encoded)


   #****************************************************************
   #
   #****************************************************************
    def comment(self):
        return self.tag('COMMENT')


   #****************************************************************
   #
   #****************************************************************
    def setComment(self, value, encoded=True):
        self.setTag('COMMENT', value, encoded)


   #****************************************************************
   #
   #****************************************************************
    def title(self):
        return self.tag('TITLE')


   #****************************************************************
   #
   #****************************************************************
    def setTitle(self, value, encoded=True):
        self.setTag('TITLE', value, encoded)


   #****************************************************************
   #
   #****************************************************************
    def year(self):
        res = self.tag('YEAR')
        if res:
            return int(self.tag('YEAR'))
        else:
            return 1900


   #****************************************************************
   #
   #****************************************************************
    def setYear(self, value):
        try:
            self.setTag('YEAR', int(value), True)
        except ValueError:
            # Try YYYY.MM.DD format
            m = re.search('(\d\d\d\d).\d\d.\d\d', value)
            if m:
                self.setTag('YEAR', int(m.group(1)), True)
                return

            # Try DD.MM.YYYY format
            m = re.search('\d\d.\d\d.(\d\d\d\d)', value)
            if m:
                self.setTag('YEAR', int(m.group(1)), True)
                return


   #****************************************************************
   #
   #****************************************************************
    def genre(self):
        return self.tag('GENRE')


   #****************************************************************
   #
   #****************************************************************
    def setGenre(self, value, encoded=True):
        self.setTag('GENRE', value, encoded)


   #****************************************************************
   #
   #****************************************************************
    def trackNum(self):
        return int(self._trackNum) + int(self._startTrackNum) - 1


   #****************************************************************
   #
   #****************************************************************
    def setTrackNum(self, value):
        self._resultFileNameCache = None
        self._resultFilePathCache = None
        self._trackNum = value
        self.emitDataChanged()


   #****************************************************************
   #
   #****************************************************************
    def startTrackNum(self):
        return self._startTrackNum


   #****************************************************************
   #
   #****************************************************************
    def setStartTrackNum(self, value):
        self._startTrackNum = value
        self.emitDataChanged()


   #****************************************************************
   #
   #****************************************************************
    def _settingsChanged(self):
        self._resultFileNameCache = None
        self._resultFilePathCache = None
        self._pattern = settings.value('OutFiles/Pattern').toString()
        self._outFormat = settings.value('OutFiles/Format').toString()


   #****************************************************************
   #
   #****************************************************************
    def safeString(self, str):
        res = str[:]
        res = res.replace('|', "-")
        res = res.replace('/', "-")
        res = res.replace('\\', "-")
        res = res.replace(':', "-")
        res = res.replace('*', "-")
        res = res.replace('?', "")
        return res


   #****************************************************************
   # %N  Number of tracks       %n  Track number
   # %a  Artist                 %A  Album title
   # %y  Year                   %g  Genre
   # %t  Track title
   #****************************************************************
    def resultFileName(self):
        if (self._resultFileNameCache):
            return self._resultFileNameCache

        pattern = self._pattern

        if (not pattern):
            pattern = QString("%a/%y - %A/%n - %t")

        perc = False
        res = ''
        for c in (pattern):
            if (perc):
                perc = False
                if   (c == '%'): res += '%'; perc = True
                elif (c == 'N'): res += "%02d" % int(self._trackSet.count())
                elif (c == 'n'): res += "%02d" % int(self.trackNum())
                elif (c == 'A'): res += self.safeString(self.album())
                elif (c == 't'): res += self.safeString(self.title())
                elif (c == 'a'): res += self.safeString(self.artist())
                elif (c == 'y'):
                    if (self.year()) :
                        res += "%02d" % int(self.year())
                elif (c == 'g'): res += self.safeString(self.genre())
                else:            res += '%' + c
            else:
                if (c == '%'):
                    perc = True
                else:
                    res += c

        ext = outputAudioFormats.current().ext
        self._resultFileNameCache = QString("%1.%2").arg(res).arg(ext)
        return self._resultFileNameCache


   #****************************************************************
   #
   #****************************************************************
    def resultFilePath(self):
        if not self._resultFilePathCache:
            self._resultFilePathCache = self._resultFilePath()

        return self._resultFilePathCache


   #****************************************************************
   #
   #****************************************************************
    def _resultFilePath(self):
        fileName = self.resultFileName()

        if fileName == '':
            return QString('')

        res = QFileInfo(fileName)

        if (res.isAbsolute()):
            return res.absoluteFilePath()

        # Add dir from trackList.outDir ===================
        outDir = settings.value('OutFiles/Directory').toString()
        if not outDir.isEmpty():
            if outDir.startsWith("~/"):
                outDir.replace(0, 1, QDir.homePath())

            res = QFileInfo(outDir + "/" + res.filePath())

        if (res.isAbsolute()):
            return res.absoluteFilePath()

        # Add dir from trackList.audioFile file ===========
        audioFileName = self._trackSet.disc().audioFileName()
        af = QFileInfo(audioFileName).dir().absolutePath()

        if (not af.isEmpty()):
            res = QFileInfo(af + "/" + res.filePath())

        return res.absoluteFilePath()


   #****************************************************************
   #
   #****************************************************************
    def discId(self):
        return self._trackSet.discId()


   #****************************************************************
   #
   #****************************************************************
    def progress(self):
        return self._progress


   #****************************************************************
   #
   #****************************************************************
    def status(self):
        return self._status


   #****************************************************************
   #
   #****************************************************************
    def setProgress(self, status, percent = -1):

        s2s = [# ------------ New statuses -----------
               # NoRun   Cancel Error  Abort   OK   Process |_____________
               [    0,     1,     1,     1,     1,     1    ], # NoRun    |
               [    1,     0,     0,     0,     0,     0    ], # Cancel   |
               [    1,     0,     0,     0,     0,     0    ], # Error    | Current
               [    1,     0,     1,     0,     0,     0    ], # Aborted  | statuses
               [    1,     0,     1,     0,     0,     0    ], # OK       |
               [    1,     0,     1,     1,     1,     1    ], # Process  |
               ]

        # Process is some is Extracting, Encoding, Queued  etc.
        procStat = len(s2s)-1

        if not s2s[min(procStat, self._status)][min(procStat, status)]:
            return

        self._status = status
        self._progress = percent


